#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/eeprom.h>

#include "UART.h"
#include "LC7980.h"
#include "vbus.h"

#define nop()	asm("nop")

volatile t_vbus_outdata vbus_outdatanew;

ISR (USART_RX) {
	uint8_t c = UDR; //Character einlesen
	
	Vbus_ProcessChar(c);
}


void VBus_Show_Values() {
	UART_PutS("---\r\n");
	uint8_t i;
	uint16_t tmp;
	for(i = 0; i < 12; i++) {			
		UART_PutS("\r\nTemp ");
		UART_PutNum(i);
		UART_PutS(":\t");
		tmp = vbus_outdatanew.temp[i];
		if(tmp & 0x8000) {
			//Zweierkomplement bei negativen Werten
			UART_PutC('-');
			tmp ^= 0xFFFF;
			tmp++;
		}
		UART_PutNum(tmp / 10);
		UART_PutC('.');
		UART_PutC(tmp % 10 + 48);
		UART_PutS("C ");
	}

	UART_PutS("\r\nEinstrahlung: ");
	UART_PutNum(vbus_outdatanew.irradiation);

	UART_PutS("\r\nSensorbruchmaske: ");
	UART_PutDHex(vbus_outdatanew.sensorbroke);
	UART_PutS("\r\nSensorkurzschlussmaske: ");
	UART_PutDHex(vbus_outdatanew.sensorshort);
	UART_PutS("\r\nSensorbenutzungsmaske: ");
	UART_PutDHex(vbus_outdatanew.sensorusage);

	for(i = 0; i < 9; i++) {
		UART_PutS("\r\nPumpe ");
		UART_PutNum(i);
		UART_PutS(": ");
		UART_PutNum(vbus_outdatanew.pump[i]);
		UART_PutS("%");
	}

	UART_PutS("\r\nRelaisbenutzungsmaske: ");
	UART_PutDHex(vbus_outdatanew.relaisusage);

	UART_PutS("\r\nFehlermaske: ");
	UART_PutDHex(vbus_outdatanew.errors);

	UART_PutS("\r\nWarnungsmaske: ");
	UART_PutDHex(vbus_outdatanew.warnings);

	UART_PutS("\r\nVersion Regler: ");
	UART_PutDHex(vbus_outdatanew.version);

	UART_PutS("\r\nZeit: ");
	UART_PutNum(vbus_outdatanew.time / 60);
	UART_PutC(':');
	UART_PutNum(vbus_outdatanew.time % 60);

	UART_PutS("\r\nUnbekannt1: ");
	UART_PutHex(vbus_outdatanew.unknown1[0]);
	UART_PutC(' ');
	UART_PutHex(vbus_outdatanew.unknown1[1]);

	UART_PutS("\r\nUnbekannt2: ");
	UART_PutHex(vbus_outdatanew.unknown2[0]);
	UART_PutC(' ');
	UART_PutHex(vbus_outdatanew.unknown2[1]);


	UART_PutS("\r\nUnbekannt3: ");
	for(i = 0; i < 4; i++) {
		UART_PutHex(vbus_outdatanew.unknown3[i]);
		UART_PutC(' ');
	}

	UART_PutS("\r\nUnbekannt4: ");
	for(i = 0; i < 4; i++) {
		UART_PutHex(vbus_outdatanew.unknown3[i]);
		UART_PutC(' ');
	}



}


int main() {
	LCD_Init();

	lcd_cursor = 200;

	UART_Init();
	sei();
	
	vbus_head.update = 1;
	vbus_head.od = &vbus_outdatanew;

	while(1) {
		if(vbus_head.valid == 1) {
			vbus_head.valid = 0;
			VBus_Show_Values();
			vbus_head.update = 1;
		}
	}
}
