#ifndef _VBUS_H
#define _VBUS_H

#define VBUS_CRC_MAGIC 0x7F

//Syncwort
#define VBUS_SYNC 0xAA

//Viessmann Vitosolic 200
#define VBUS_SRC 0x7321
//DFA (Datenfernanzeige)
#define VBUS_DST 0x0010

#define VBUS_FRAME_COUNT 18

//Wenn das Frame ungltig ist, wird folgender Wert in die Ausgabe geschrieben
#define VBUS_INVALID_FRAME 0xFF

//Protokollversionen
//momentan wird nur Version 1.0 untersttzt
#define VBUS_PROTO_10 0x10
#define VBUS_PROTO_20 0x20
#define VBUS_PROTO_30 0x30

//Lnge des Headers
#define VBUS_HEADER_LENGTH 10

//Anzahl der Nutzdaten, die ein V1.0-Paket trgt
#define VBUS_10_FRAME_PAYLOAD 4

//Gesamtgre eines V1.0-Pakets (Nutzdaten + Septett + Checksum)
#define VBUS_10_FRAME_LENGTH (VBUS_10_FRAME_PAYLOAD + 2)


#define VBUS_DEBUG 0

#if VBUS_DEBUG == 1
	#define VBUS_DEBUG_C(c) LCD_PutC(c)
	#define VBUS_DEBUG_S(s) LCD_PutS(s)
	#define VBUS_DEBUG_I(i) LCD_PutNum(i)
	#define VBUS_DEBUG_H(h) LCD_PutHex(h)
	#define VBUS_DEBUG_DH(h) LCD_PutDHex(h)
	#define VBUS_DEBUG_BREAK() LCD_ClearHome()
#else
	#define VBUS_DEBUG_C(str)
	#define VBUS_DEBUG_S(str)
	#define VBUS_DEBUG_I(str)
	#define VBUS_DEBUG_H(str)
	#define VBUS_DEBUG_DH(str)
	#define VBUS_DEBUG_BREAK()
#endif



/* Frameinhalt eines VBus 1.0-Pakets
 * offset gehrt nicht zum Paketinhalt
 */
typedef struct {
	uint8_t data[4];
	uint8_t septett;
	uint8_t checksum;
	uint8_t offset;
} t_vbus10_frame;

/* Empfangene Daten vom Gert
 * Hier: spezifisch fr Viessmann VitoSolic 200
 * Die Infos bzgl. Lnge, Position, etc. knnen den XML-Dateien 
 * %ProgramFiles%\RESOL\ServiceCenterFull\eclipse\plugins\de.resol.servicecenter.vbus.<Device>\VBusSpecification<Device>.xml
 * entnommen werden.
 * Das Resol ServiceCenter (RSC) gibt's kostenlos unter http://www.resol.de
 */
typedef struct {
	int16_t  temp[12];    //Offset  0-22
	uint16_t irradiation; //Offset 24-25
	uint8_t  unknown1[2]; //Offset 26-27
	uint32_t impulse[2];  //Offset 28-35
	uint16_t sensorbroke; //Offset 36-37
	uint16_t sensorshort; //Offset 38-39
	uint16_t sensorusage; //Offset 40-41
	uint8_t  unknown2[2]; //Offset 42-43
	uint8_t  pump[10];    //Offset 44-53
	uint8_t  unknown3[4]; //Offset 54-57
	uint16_t relaisusage; //Offset 58-59
	uint16_t errors;      //Offset 60-61
	uint16_t warnings;    //Offset 62-63
	uint16_t version;     //Offset 64-65
	uint16_t time;        //Offset 66-67
	uint8_t  unknown4[4]; //rest
} t_vbus_outdata;

/* Kopfinformationen des VBus-Pakets
 * sync, offset und currframe gehren nicht zum Paket, 
 * sondern sind Informationen, die beim Decodieren bentigt werden
 */
typedef struct {
	uint16_t dest;
	uint16_t source;
	uint8_t version; //eigentlich nur 4 bit
	uint16_t command;
	uint8_t frames;
	uint8_t checksum;
	uint8_t currframe;
	volatile t_vbus_outdata* od;
	unsigned sync:1;
	unsigned offset:7;
	unsigned update:1;
	unsigned valid:1;
} t_vbus_head;

extern volatile t_vbus_head vbus_head;
extern t_vbus10_frame vbus_frame;

extern void Vbus_ProcessChar(uint8_t);
extern uint8_t VBus_CalcCrc(const unsigned char*, uint8_t, uint8_t);

#endif
